using System;
using System.Collections.Generic;
using System.Diagnostics.CodeAnalysis;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using MobileChargingStation.Lib.Models;
using UsbSimulator;
using MobileChargingStation.Lib.Utils;

namespace MobileChargingStation.Lib.Controller
{
public class ChargeControl
    {
        private readonly IUsbCharger _usbCharger;
            private readonly IDisplay _display; 


        public ChargeControl(IUsbCharger usbCharger,IDisplay display )
        {
            _usbCharger = usbCharger;
            _usbCharger.CurrentValueEvent += HandleCurrentValueChanged;
            _display = display; 
           
            


        }

        private void HandleCurrentValueChanged(object? sender, CurrentEventArgs e)
        {
            //Console.WriteLine($"USB Current changed: {e.Current} mA");

             _display.ShowMessage($"USB Current changed: {e.Current} mA");
             Thread.Sleep(1000);
        }

        public void StartCharging()
        {
            if (_usbCharger.Connected)
            {
                _display.ShowMessage("Starting USB charging...");
                _usbCharger.StartCharge();
            }
            else
            {
                _display.ShowMessage("No device connected, cannot start charging.");
            }
        }

        public void StopCharging()
        {
            _display.ShowMessage("Stopping USB charging...");
            _usbCharger.StopCharge();
        }

        public bool IsConnected()
        {
            return _usbCharger.Connected;
        }
    }
}
