using System;
using System.Collections.Generic;
using System.Diagnostics.CodeAnalysis;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Threading;

using MobileChargingStation.Lib.Models;
using UsbSimulator;
using MobileChargingStation.Lib.Utils;

namespace MobileChargingStation.Lib.Controller
{
    public class StationControl
    {
        private readonly IDoor _door;
        private readonly ChargeControl _chargeControl;
        private readonly IRfidReader _rfidReader; 

        private readonly ILogger _logger; 

        private readonly IDisplay _display;

        private int _currentRfidId;

        public StationControl(IDoor door, IUsbCharger usbCharger, IRfidReader rfidReader, IDisplay display, ILogger logger = null) 
        {
            _logger = logger ?? new Logger(); 
            _door = door;
            _chargeControl = new ChargeControl(usbCharger, display); 
            _rfidReader = rfidReader;
            _door.DoorChangedEvent += HandleDoorChanged;
            _display = display; 
        }


        private void HandleDoorChanged(object? sender, DoorChangedEventArgs e)
        {
            HandleDoorStatus(e.DoorOpen, e.DoorUnlocked, e.DoorAvailable, e.recharging);
        }

        private void HandleDoorStatus(bool doorOpen, bool doorUnlocked, bool doorAvailable, bool recharging)
        {
            switch ((doorOpen, doorUnlocked, doorAvailable, recharging))
            {
                case (false, true, true, false):
                    _display.ShowMessage("Klar til brug opdateret til displaylogger :)");
                    break;
            // efter vi har kaldt open door, så vil dette komme frem 
                case (true, false, false, false):
                    _display.ShowMessage("tislut svinet");
                    break;


                // så lukker duden døren igen det her er close door
                    case (false, true, false, false):
                    _display.ShowMessage("indlæs RFID");
                    bool rfidDetected = _rfidReader.DetectRfid(_currentRfidId);
                    if (rfidDetected)
                    {   
                        LockDoor();
                    
                    
                    }
                    else 
                    {
                        _display.ShowMessage("RFID detection failed. Access denied.");
                        FailedStartForfra();
                    }
                    break;


                    case (false, false, false, false):
                    bool rfidDetected2 = _rfidReader.DetectRfid(_currentRfidId);
                    if (rfidDetected2)
                    {
                        if (_chargeControl.IsConnected())
                        {
                            _chargeControl.StartCharging();
                           
                           Thread.Sleep(2000);
                            ChargingLockedDoor();
                            

                            

                           

                        }
                        else
                            {
                                _display.ShowMessage("Usb ikke tilsuttet... døren åbnes op");
                             
                                FailedStartForfra();
                            }
                        

                    }
                    else
                    {
                            _display.ShowMessage("RFID detection failed. Access denied.");
                    }
                    break;

                case (false, false, false, true):
                    _display.ShowMessage("Scan your RFID");
                    bool rfidDetectedOut = _rfidReader.DetectRfid(_currentRfidId);
                    if (rfidDetectedOut)
                    {
                        _chargeControl.StopCharging();
                        UnlockDoor();
                    }
                    else
                    {
                        _display.ShowMessage("RFID detection failed. Access denied.");
                    }
                    break;

                
            }
        }

        public void LockDoor()
        {
           
            _logger.Log("logDoorLocked(" + _currentRfidId + ")");
            _door.SetDoorStatus(false, false, false, false);
        }

        public void UnlockDoor()
        {
           _logger.Log("logDoorUnlocked(" + _currentRfidId + ")");
            _door.SetDoorStatus(false, true, true, false);
        }

        public void FailedStartForfra()
        {
          
            _door.SetDoorStatus(false, true, true, false);
        }

        public void OpenDoor()
        {
            _display.ShowMessage("Simulering ad døren åbner.");
            _door.SetDoorStatus(true, false, false, false);
        }

        public void ChargingLockedDoor()
        {
            
            _door.SetDoorStatus(false, false, false, true);
        }


        public void CloseDoor()
        {
            _display.ShowMessage("simulering af døren lukker");
            _door.SetDoorStatus(false, true, false, false);
        }





        public void SetRfidId(int rfidID)
        {
            _currentRfidId = rfidID;
        }

        public void DetectRfid(int rfidID)
        {
            _rfidReader.DetectRfid(rfidID);
        }
    }
}