using System;

namespace MobileChargingStation.Lib.Models
{
    public class RfidReader : IRfidReader
    {
        public event EventHandler<RfidDetectedChangedEventArgs>? RfidChangedEvent;

        private readonly int[] _validRfidIds = { 123, 124 }; // valide rfid id'er, grunden til vi har to, er for at teste det, der vil jo naturligvis være flere i virkeligheden

        public bool DetectRfid(int rfidID)
        {
            int attempts = 0;
            const int maxAttempts = 3;

            while (attempts < maxAttempts)
            {
                if (Array.Exists(_validRfidIds, id => id == rfidID)) 
                {
                    OnRfidChanged(new RfidDetectedChangedEventArgs { RfidID = rfidID });
                    return true;
                }
                else
                {
                    Console.WriteLine("RFID not recognized, try again");
                    attempts++;
                    if (attempts == maxAttempts)
                    {
                        Console.WriteLine("Maximum attempts reached. Access denied.");
                        Thread.Sleep(300);
                        
                    }
                }
            }
            return false;
        }

        protected virtual void OnRfidChanged(RfidDetectedChangedEventArgs e)
        {
            RfidChangedEvent?.Invoke(this, e);
        }
    }

    public class RfidDetectedChangedEventArgs : EventArgs
    {
        public int RfidID { get; set; }
    }

    public interface IRfidReader
    {
        event EventHandler<RfidDetectedChangedEventArgs> RfidChangedEvent;
        bool DetectRfid(int rfidID);
    }
}
