using System;
using System.IO;

namespace MobileChargingStation.Lib.Utils
{
    public interface ILogger
    {
        void Log(string message);
    }

    public class Logger : ILogger
    {
        private readonly ILogger _fileLogger; 

        public Logger()
        {
           
            string fullPath = Path.GetFullPath(Path.Combine(Directory.GetCurrentDirectory(), "Logs", "rfid_log.txt"));
            _fileLogger = new FileLogger(fullPath);
        }
        
        public Logger(ILogger fileLogger)
        {
            _fileLogger = fileLogger;
        }

        public void Log(string message)
        {
            _fileLogger.Log(message);
        }
    }

    public class FileLogger : ILogger
    {
        private readonly string _logFilePath;

        public FileLogger(string logFilePath)
        {
            if (string.IsNullOrWhiteSpace(logFilePath))
            {
                throw new ArgumentException("The log file path cannot be null or empty.", nameof(logFilePath));
            }

            var directory = Path.GetDirectoryName(logFilePath);
            
            if (directory == null || !Path.IsPathFullyQualified(directory))
            {
                throw new ArgumentException("The log file path is invalid or not fully qualified.", nameof(logFilePath));
            }

            if (!Directory.Exists(directory))
            {
                Directory.CreateDirectory(directory); 
            }

            _logFilePath = logFilePath;
        }

        public void Log(string message)
        {
           
            string timestampedMessage = $"[{DateTime.Now:yyyy-MM-dd HH:mm:ss}] {message}";
            File.AppendAllText(_logFilePath, timestampedMessage + Environment.NewLine);
        }
    }
}
