## Introduktion
Dokumentaton for udarbejdelsen af systemet MobileChargingStation handin-2 i Software test.
Systemet er udviklet af gruppe 7 som består af:

**SWTF25 7**

---

## Gruppe Medlemmer

| Studerende         | Studienummer |
|--------------------|-------------|
| Fillip Fontao      | 202109701   |
| Jakob Alsina Lund  | 202308406   |

---


## Design og implementering
I det nedestående afsnit vil vi gå igennem designet og implementering af interfaces og event-driven programmering,  her vil vi gennemgå fordele og ulemper, og hvordan vi har brugt dem i udviklingen af MobileChargingStation systemet.
Vi har bygget systemet omkring en loose coupling, hvor de forskellige komponenter kommunikerer gennem events. Dette sker i Idoor, IRFIDReader, og UsbChargeSimulator. 
Hvorefter vi har 2 controlers, som styre hvad disse events bruges til.

## Klassediagrammer

Vi har udarbejdet flere klassediagrammer for at give et struktureret overblik over vores løsning og samtidig støtte op om vores testbare design. Vi vil i vores dokumentation gå i dybden med det overordnede, Chargecontroller og StationController klasser, herudover vi vise et sd diagram, som viser hændelse forløb for brug af opladeren, og forklare hvordan klasserne inteagere med hinanden.

### Overordnet Klassediagram

![alt text](billeder/OverordnetNy.drawio.png)

Det overordnede klassediagram viser samspillet mellem alle hovedkomponenterne i systemet. Diagrammet visualiserer, hvordan StationControl fungerer som central styringsenhed, der interagerer med de andre klasser som ChargeControl, Door, RfidReader, UsbChargerSimulator, ConsoleLogger og display. Desuden fremgår det tydeligt, hvordan forskellige interfaces som IDoor, IRfidReader og IUsbCharger bruges til at abstrahere implementeringen, hvilket gør systemet fleksibelt og let at teste.

Dette diagram giver et hurtigt overblik over hele systemets struktur, afhængigheder og informationsflow.



Derudover har vi udarbejdet 7 detaljerede klassediagrammer – ét for hver vigtig klasse. Vi har dog valgt at kun fokusere på Stationcontroller og ChargeController, og derfor kun har dem med i dokumentationen.


### ChargeControl Klassediagram

 Diagrammet viser, hvordan denne klasse styrer opladningen via afhængigheden til IUsbCharger og Display hændelser med IDisplay. Den håndterer USB Current-events og har ansvar for at starte og stoppe opladning.

![alt text](billeder/CCnY.drawio.png)


### StationControl Klassediagram

Diagrammet illustrerer, hvordan StationControl styrer efter door events,, opladning og RFID-læsning. Dens afhængigheder til IDoor, ChargeControl, IRfidReader, ILogger, Idisplay gør den central for flowstyringen.

![alt text](billeder/Nystation.drawio.png)



Vores designvalg har haft et klart fokus på at sikre løsningen er testbar, fleksibel og overskuelig:

Ved at abstrahere funktionalitet som Door, RFID-læser, oplader,  display og logger bag interfaces, kan vi let udskifte disse med mocks eller simuleringer under test. Det gør det muligt at skrive enhedstests uden afhængighed af hardware eller brugerinput.

Komponenterne Door og UsbChargerSimulator,Rfidreader udsender events ved tilstandsændringer, hvilket gør systemet løst koblet og let at simulere i tests, dette medfører også at hvert

klasse har ét klart ansvarsområde, hvilket gør dem lettere at teste individuelt, men også koden nemmere at forstå, da det nemt at finde, hvor de forskellige metoder er implementeret.

Alle afhængigheder injiceres via konstrukteren, hvilket yderligere forenkler testopsætning.

## Sekvensdiagram

![Sekvensdiagram](billeder/SdTilSoftwareTest-SideEndelig.drawio.png)

# Sekvensbeskrivelse af Ladeskabssystemet
Herunder beskrives kort de events, der sker i koden og sekvensdiagrammet.
Vi har delt det op i forskellige sekvenser, så det gør det nemmere at forklare, da det er et ret langt sd dia. Alternativt, kunne vi have delt sd dia op i flere, men vi har valgt at lave en beskrivende tekst istedet.

## Låsning og Start af Opladning



- Bruger åbner døren. **Door** ændrer sin tilstand og udløser et `DoorChangedEvent`.
- Bruger tilslutter telefonen. **USBCharger** registrerer en tilsluttet enhed.
- Bruger lukker døren. **Door** ændrer sin tilstand og udløser endnu et `DoorChangedEvent`.
- Bruger scanner RFID-tag. **RFIDReader** validerer RFID og udløser et `RfidChangedEvent`.

### Hvis RFID er ukorrekt:

- **RFIDReader** beder brugeren om at prøve igen.
- `attempts` tælles op. Når `attempts == 3`:
  - **RFIDReader** venter 5 minutter (sleep).
  - **StationControl** kalder `FailedStartforfra`, som:
    - Starter sekvensen forfra, eller
    - Giver brugeren mulighed for at tage telefonen igen.
  - Ventetid simuleres med `Thread.Sleep()` 

### Hvis RFID er korrekt:

- **StationControl** logger i en `.txt`-fil via `LogFile.LogDoorLocked()` og instruerer **Door** til at låse. Dette udløser et `DoorChangedEvent`.
- **StationControl** kalder `isConnected()` på **ChargeControl**, som tjekker tilslutningen via **USBCharger**.
  - Hvis ikke tilsluttet:
    - `FailedStartforfra` kaldes, og sekvensen starter forfra.
  - Hvis tilsluttet:
    - **ChargeControl** starter opladningen via **USBCharger**.
    - **StationControl** låser døren og simulerer opladning med `Thread.Sleep(5000)`.

## Oplåsning og Afhentning af Telefon

- Bruger scanner RFID-tag igen. **RFIDReader** validerer og udløser et `RfidChangedEvent`.

### Hvis RFID er ukorrekt:

- **StationControl** starter et loop. Hvis `attempts == 3`:
  - **RFIDReader** venter 5 minutter (sleep).
  - `attempts` nulstilles.
  - Brugeren kan prøve igen.

### Hvis RFID er korrekt:

- **StationControl**:
  - Stopper opladningen via **ChargeControl**.
  - Logger oplåsning i `.txt`-fil med `LogFile.LogDoorUnlocked()`.
  - Instruerer **Door** til at låse op.

Til sidst starter **StationControl** et event, der:

- Sætter **Door** tilbage til starttilstand og udløser et `DoorChangedEvent`.
- Markerer skabet som ledigt, så brugeren kan tage sin telefon.

## Statemachine

For at illustrere systemets opførsel har vi udarbejdet en state machine, som viser de forskellige tilstande og overgange i løsningen. Systemet håndterer interaktionen mellem bruger, RFID-scanning, dørstatus og opladning.

![alt text](billeder/image.png)

## Refleksion over designvalg

I udviklingen af systemet har vi haft særligt fokus på testbarhed og fleksibilitet. For at muliggøre test uden adgang til fysisk hardware valgte vi at indkapsle de hardwareafhængige dele gennem interfaces. Det gjorde det nemt at erstatte dem med fakes og mocks, hvilket resulterede i stabile og kontrollerbare tests. Selvom denne tilgang krævede lidt ekstra kode og opsætning, vurderer vi, at det har været en klar fordel i forhold til senere vedligeholdelse.

Vi gør desuden brug af et event-baseret design, og observer-pattern, for at opnå løs kobling mellem komponenterne. Det gav os et system, hvor f.eks. visning og logning kunne ske uafhængigt af den centrale styring. Dette design har gjort systemet mere modulært og testvenligt. Dog har vi erfaret, at event-håndtering kan være sværere at overskue og debugge, især den måde vi har lavet det med cases uden navn, det tog en del tid og tålmodighed, at få det til at virke, da vi hele tiden skulle kigge frem og tilbage, ift hvad disse states gjorde. Dette vil vi undgå næste gang ved at gøre brug af enum states, da dette går lidt imod "senere vedligeholdelse" Derfor kræver denne tilgang, at man er omhyggelig og designer tingene bedre til næste gang.

##  Teststrategier 
Vi vil kort forklare om teststrategierne, og herefter forklare hvordan de bruges i vores test.

# Equivalence Partitioning (EP)

Equivalence Partitioning handler om at opdele input i logiske grupper

Et eksempel er at et system tillader input mellem 1 og 100.

- Partition 1: Tal under 1 her vil man forvente fejle
- Partition 2: Tal mellem 1 og 100 her vil man forvente at testen godkendes godkendes
- Partition 3: Tal over 100 her vil en fejl test forventes

# Boundary Value Analysis (BVA)

Boundary Value Analysis fokuserer på at teste lige ved grænserne for input. 

hvis vi tager udgangspunkt i det eksempel vi brugte ved EP

- Test 0 lige under minimum her vil man forvente fejl
- Test 1 dette er minumum, så her vil man forvente at testen udføres korrekt
- Test 100 dette er maksimum, og man vil så forvente at testen udføres korrekt.
- Test 101 lige over maksimum her vil man forvente at testen fejler



# Zombie Testing

Zombie Testing tester at systemet ikke foretager sig noget, når det ikke burde, så feks. hvis vi endnu engang tager udgangspunkt i overstående
hvis tallet er 0, og der skal ske noget mellem 1 og 100, så skal der ikke ske noget, hvis der skete noget ville testen faile.
så zombie testing tester outliers.

Forventet resultat: Systemet forbliver inaktivt og gør ingenting.


##  Teststrategier brugt i projektet.
Herunder vil vi komme med konkrete eksempler, hvor vi har brugt test strategierne.

# EP
Vi tester RFid i 2 partioner, 
test 1: Gyldige RFID-ID, her forventer vi, at systemet accepterer ID'et, returnerer true og udløser et event. 

test2: Ugyldige RFID-ID, her forventer vi, at systemet afviser ID'et, returnerer false og ikke udløser nogen hændelse.

```csharp
[Test]
        public void DetectRfid_ValidId_ReturnsTrue_AndRaisesEvent()
        {
           
            bool eventRaised = false;
            RfidDetectedChangedEventArgs? eventArgs = null;
            int validRfidID = 123; 

            _uut!.RfidChangedEvent += (sender, args) =>
            {
                eventRaised = true;
                eventArgs = args;
            };

           
            bool result = _uut.DetectRfid(validRfidID);

            
            Assert.That(result, Is.True);
            Assert.That(eventRaised, Is.True);
            Assert.That(eventArgs, Is.Not.Null);
            Assert.That(eventArgs!.RfidID, Is.EqualTo(validRfidID));
        }

        [Test]
        public void DetectRfid_InvalidId_ReturnsFalse_AndDoesNotRaiseEvent()
        {
           
            bool eventRaised = false;
            int invalidRfidID = 456; 

            _uut!.RfidChangedEvent += (sender, args) =>
            {
                eventRaised = true;
            };

            bool result = _uut.DetectRfid(invalidRfidID);

           
            Assert.That(result, Is.False);
            Assert.That(eventRaised, Is.False);
        }

```
# Bva
 Der er en grænse for hvor mange gange et ugyldigt RFID-kort kan scannes.


Test 1: Når grænsen for forsøg er nået Vi tester, at systemet reagerer korrekt, når en bruger har scannet et ugyldigt kort det maksimale antal gange, skal døren ikke låses op.

Test 2: Flere forsøg med ugyldigt kort Vi tester også, at systemet forbliver stabilt, selv hvis brugeren bliver ved med at prøve med det samme ugyldige kort. Heller ikke her må døren låses op, og der må ikke udløses nogen handlinger.


Disse tests sikrer, at vores system er sikkert og stabilt, selv når brugere prøver at "presse" dets grænser.

```csharp
        public void DetectRfid_MaxAttemptsReached_ReturnsFalse()
        {
            int invalidRfidID = 456;
            int maxAttempts = 3; 
            
        
            for (int i = 0; i < maxAttempts - 1; i++)
            {
                _uut!.DetectRfid(invalidRfidID);
            }
            
        
            bool result = _uut!.DetectRfid(invalidRfidID);
            
            Assert.That(result, Is.False);
        }

        [Test]
        public void DetectRfid_InvalidId_MultipleAttempts_StillReturnsFalse()
        {
            int invalidRfidID = 456;
            int maxAttempts = 3; 
            bool eventRaised = false;

            _uut!.RfidChangedEvent += (sender, args) =>
            {
                eventRaised = true;
            };
            
            
            for (int i = 0; i < maxAttempts + 2; i++)
            {
                _uut!.DetectRfid(invalidRfidID);
            }
            
           
            bool result = _uut!.DetectRfid(invalidRfidID);
            
            Assert.That(result, Is.False);
            Assert.That(eventRaised, Is.False);
        }

```

### ZOMBIE-strategien
Her gør vi i testen brug af zoombie, som står for manny, så dermed tester vi flere hændelse på engang, og dermed sikre, at den systemet opfører som forventet

```csharp
[Test]
        public void DetectRfid_MultipleValidIds_RaisesEventEachTime()
        {
            int validRfidID1 = 123;
            int validRfidID2 = 124;
            int eventCount = 0;

            _uut!.RfidChangedEvent += (sender, args) =>
            {
                eventCount++;
            };

            _uut.DetectRfid(validRfidID1);
            _uut.DetectRfid(validRfidID2);

            Assert.That(eventCount, Is.EqualTo(2));
        }

```



#  Intern arbejde

Vi valgte at fordele arbejdet ud fra en kombination af vores individuelle styrker og interesser, samt de krav der var til opgaven. Fra projektets start lavede vi en fælles plan, hvor vi identificerede de vigtigste moduler og komponenter (controllers, modeller, test osv.) Dette gjorde vi ved at lave klasse diagrammerne, herefter lavede vi sekvens diagramet, og så fordelte implementationen imellem os.

Eksempelvis tog én person ansvaret for udviklingen af StationControl og dens logik, mens en anden havde fokus på ChargeControl og integrationen med USB-laderen. Vi koordinerede løbende, så opgaverne blev løst parallelt, men stadig integreret. Derudover satte vi tid af til at skrive tests i fællesskab og lave code reviews på hinandens arbejde.

Fordele:

Vi kunne arbejde effektivt parallelt uden at vente på hinanden.
Det gav hver især mulighed for at tage ejerskab over en del af systemet.

Ulemper:

Når vi arbejdede hver for sig, kunne der opstå integration issues, som krævede ekstra tid.
Risiko for at én persons arbejde blev afhængig af, at den andens modul blev færdigt til tiden.

##  Brug af repo og CI/CD

Vi brugte et delt GitLab repository med CI-pipeline, der kørte vores unit tests og coverage ved hvert commit. Det gjorde det muligt at spotte fejl hurtigt og holde stabil kode.
![alt text](billeder/image-1.png)

Rapporten viser, at vi har opnået 100% line og method coverage, samt 97.6% branch coverage på hele kodebasen. Alle klasser og filer er fuldt dækket af tests, hvilket bekræfter, at vores teststrategi har været ok. Måden vi har arbejdet med det, er at ved hvert push af nye test, har vi været inde og se hvor coverage lagde på, hvorefter vi har været inde på coverage raporten, og tjekke hvor vi manglede coverage. Herefer har vi lavet flere test til at til sidst nå en  line coverage på 100%

Derudover implementerede vi også branch rules:
- Ingen push direkte til main
- Kun merge fra egen branch
- Merge kun hvis tests er godkendte

Vi brugte NUnit som test framework og NSubstitute som mocking framework til at skabe testbare versioner af vores interfaces. NSubstitute blev anvendt bredt i vores unit tests til at isolere systemkomponenter og kontrollere deres opførsel uden afhængighed af faktisk implementeret logik.

Eksempler hvor vi anvendte NSubstitute:

- I ChargeControlTest.cs mockede vi IUsbCharger, IDisplay og ILogger, så vi kunne verificere:
  - om StartCharge() og StopCharge() blev kaldt korrekt
  - om displayet viste de rigtige beskeder afhængigt af strømtilstand
  - at logging foregik som forventet uden at bruge en ægte logfil

- I StationControlTests.cs brugte vi NSubstitute til at mocke:
  - IDoor  så vi kunne kontrollere om døren blev låst/ulåst korrekt
  - IRfidReader og ILogger for at tjekke om hændelser blev håndteret og logget korrekt
  - IDisplay  for at verificere om brugeren fik den rigtige feedback (beskeder) ved forskellige scenarier

- I DoorTest.cs og RfidReaderTest.cs blev NSubstitute brugt til at kontrollere event-subscription og hændelsesudløsning.

Ved at bruge NSubstitute kunne vi køre vores tests hurtigt og uafhængigt af den fysiske hardware og samtidig fokusere på hver enkelt klasses opførsel i isolation. Det har givet os stor fleksibilitet og pålidelig testdækning.
